import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_progress_hud/flutter_progress_hud.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:intl/intl.dart';
import 'package:unit2/bloc/offline/offline_passo/land/land_property_assessment/land_property_assessment_bloc.dart';
import 'package:unit2/screens/offline/passo/land/edit/AddPropertyAssessmentEditModal.dart';

import '../../../../../model/passo/land_property_assessment.dart';
import '../../../../../utils/alerts.dart';
import '../../../../../utils/text_container.dart';
import '../../../../../widgets/error_state.dart';
import '../../../../../widgets/passo/custom_button.dart';

class LandPropertyAssessmentEditPageOffline extends StatefulWidget {
  Function PrevBtn;
  Function NextBtn;
  final int faasId;
  LandPropertyAssessmentEditPageOffline(
      this.PrevBtn, this.NextBtn, this.faasId);
  @override
  _LandPropertyAssessmentEditPageOffline createState() =>
      _LandPropertyAssessmentEditPageOffline();
}

class _LandPropertyAssessmentEditPageOffline
    extends State<LandPropertyAssessmentEditPageOffline> {
  //  double _totalMarketValue(items) {
  //   double total = 0;
  //   items.forEach((row) {
  //     total += double.parse(row);
  //   });
  //   return total;
  // }

  void deleteItem(int itemId) {
    context
        .read<LandPropertyAssessmentBloc>()
        .add(DeleteLandPropertyAssessment(id: itemId));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: ProgressHUD(
        padding: const EdgeInsets.all(24),
        backgroundColor: Colors.black87,
        indicatorWidget: const SpinKitFadingCircle(color: Colors.white),
        child: BlocConsumer<LandPropertyAssessmentBloc,
            LandPropertyAssessmentState>(listener: (context, state) {
          // if (state is LandPropertyAssessmentLoading) {
          //   final progress = ProgressHUD.of(context);
          //   progress!.showWithText("Please wait...");
          // }
          // if (state is LandPropertyAssessmentLoaded) {
          //   final progress = ProgressHUD.of(context);
          //   progress?.dismiss();
          // }
          // if (state is LandPropertyAssessmentErrorState) {
          //   final progress = ProgressHUD.of(context);
          //   progress?.dismiss();
          // }
        }, builder: (context, state) {
          final state = context.watch<LandPropertyAssessmentBloc>().state;
          if (state is LandPropertyAssessmentLoaded) {
            return Column(
              children: [
                Expanded(
                    child: SingleChildScrollView(
                  child: Padding(
                    padding: const EdgeInsets.all(15.0),
                    child: Column(
                      children: [
                        Container(
                          margin: const EdgeInsets.only(
                              left: 0, top: 20, right: 0, bottom: 10),
                          child: const Text('PROPERTY ASSESSMENT',
                              style: TextStyle(
                                  fontWeight: FontWeight.bold, fontSize: 18),
                              textAlign: TextAlign.left),
                        ),
                        Align(
                          alignment: Alignment.topRight,
                          child: ElevatedButton(
                            style: ElevatedButton.styleFrom(
                              backgroundColor: Colors.red,
                            ),
                            onPressed: () {
                              context
                                  .read<LandPropertyAssessmentBloc>()
                                  .add(ShowLandPropertyAssessment());
                            },
                            child: Row(
                              mainAxisSize: MainAxisSize.min,
                              children: [
                                const Text('ADD ITEM'), // <-- Text
                                const SizedBox(
                                  width: 5,
                                ),
                                const Icon(
                                  // <-- Icon
                                  Icons.add,
                                  size: 24.0,
                                ),
                              ],
                            ),
                          ),
                        ),
                        SingleChildScrollView(
                            scrollDirection: Axis.horizontal,
                            child: DataTable(
                                // ignore: prefer_const_literals_to_create_immutables
                                columns: [
                                  const DataColumn(
                                    label: Text('Actual Use'),
                                  ),
                                  const DataColumn(
                                    label: Text('Market Value'),
                                  ),
                                  const DataColumn(
                                    label: Text('Assessment Level'),
                                  ),
                                  const DataColumn(
                                    label: Text('Assessed Value'),
                                  ),
                                  const DataColumn(
                                    label: Text('Action'),
                                  )
                                ],
                                rows:
                                    state.landPropertyAssessment.map((dataRow) {
                                  return DataRow(
                                    cells: [
                                      DataCell(Text(dataRow.actualUse ?? "")),
                                      DataCell(Text(NumberFormat.currency(
                                        locale: 'en-PH',
                                        symbol: "₱",
                                      ).format(
                                          double.parse(dataRow.marketval!)))),
                                      DataCell(Text(dataRow.assessmentLevel ??
                                          "0" + '%')),
                                      DataCell(Text(NumberFormat.currency(
                                        locale: 'en-PH',
                                        symbol: "₱",
                                      ).format(double.parse(
                                          dataRow.assessedValue!)))),
                                      DataCell(Row(
                                        children: [
                                          InkWell(
                                            child: Container(
                                              height: 30,
                                              width: 30,
                                              decoration: BoxDecoration(
                                                shape: BoxShape.circle,
                                                color: Colors.red,
                                              ),
                                              child: Icon(
                                                Icons.delete,
                                                color: Colors.white,
                                                size: 20.0,
                                              ),
                                            ),
                                            onTap: () {
                                              confirmAlertWithCancel(
                                                  context,
                                                  () => deleteItem(dataRow.id!),
                                                  () => null,
                                                  'Delete Item?',
                                                  "Are you sure you want to delete this item?");
                                            },
                                          ),
                                        ],
                                      ))
                                    ],
                                  );
                                }).toList()))
                      ],
                    ),
                  ),
                )),
                // Padding(
                //   padding: const EdgeInsets.only(left: 20.0, right: 20.0),
                //   child: Row(
                //     mainAxisAlignment: MainAxisAlignment.spaceBetween,
                //     children: [
                //       Text(
                //         'Total',
                //         style: TextStyle(fontWeight: FontWeight.bold, fontSize: 15),
                //       ),
                //       Text(
                //         NumberFormat.currency(locale: 'en-PH', symbol: "₱")
                //             .format('1.0'),
                //         style: TextStyle(fontWeight: FontWeight.bold, fontSize: 15),
                //       )
                //     ],
                //   ),
                // ),
                Padding(
                  padding: const EdgeInsets.all(15.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      CustomButton(
                        icon: const Icon(Icons.chevron_left_rounded,
                            color: Colors.white),
                        onPressed: () {
                          {
                            widget.PrevBtn();
                          }
                          ;
                        },
                      ),
                      CustomButton(
                        icon: const Icon(Icons.chevron_right_rounded,
                            color: Colors.white),
                        onPressed: () {
                          {
                            widget.NextBtn();
                          }
                          ;
                        },
                      )
                    ],
                  ),
                ),
              ],
            );
          }
          if (state is LandPropertyAssessmentDeletedState) {
            if (state.success) {
              WidgetsBinding.instance.addPostFrameCallback((_) {
                successAlert(context, "Deletion Successful",
                    "Extra item has been deleted successfully", () {
                  Navigator.of(context).pop();
                  context.read<LandPropertyAssessmentBloc>().add(
                      LoadLandPropertyAssessmentEdit(
                          landPropertyAssessment: <LandPropertyAssessment>[],
                          id: widget.faasId));
                });
              });
            }
          }
          if (state is ShowLandPropertyAssessmentcreen) {
            return ConstrainedBox(
              constraints: BoxConstraints(maxHeight: 1000.0),
              child: AlertDialog(
                insetPadding: EdgeInsets.symmetric(
                  horizontal: 20.0,
                  vertical: 10.0,
                ),
                title: Text(
                  'ADD PROPERTY ASSESSMENT',
                  textAlign: TextAlign.center,
                ),
                content: Column(
                  mainAxisSize: MainAxisSize.min,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Container(child: AddPropertyAssessmentEditOfflineModal())
                  ],
                ),
              ),
            );
          }
          if (state is LandPropertyAssessmentErrorState) {
            return SomethingWentWrong(
              message: onError,
              onpressed: () {
                context.read<LandPropertyAssessmentBloc>().add(
                    LoadLandPropertyAssessmentEdit(
                        landPropertyAssessment: const <LandPropertyAssessment>[],
                        id: widget.faasId));
              },
            );
          }

          return Container();
        }),
      ),
    );
  }
}
