import 'dart:convert';
import 'dart:developer';
import 'dart:io';

import 'package:http/http.dart' as http;
import 'package:unit2/model/passo/land_property_boundaries.dart';
import 'package:unit2/utils/request.dart';
import 'package:unit2/utils/urls.dart';

class LandBoundariesService {
  static final LandBoundariesService _instance = LandBoundariesService();
  static LandBoundariesService get instance => _instance;

  String xClientKey = "unitK3CQaXiWlPReDsBzmmwBZPd9Re1z";
  String xClientKeySecret = "unitcYqAN7GGalyz";

  Future<List<LandPropertyBoundaries>> fetch() async {
    String path = Url.instance.getLandPropertyBoundaries();
    Map<String, String> headers = {
      'Content-Type': 'application/json; charset=UTF-8',
      'X-Client-Key': xClientKey,
      'X-Client-Secret': xClientKeySecret
    };
    try {
      http.Response response = await Request.instance
          .getRequest(param: {}, path: path, headers: headers);

      if (response.statusCode == 200) {
        final List result = jsonDecode(response.body)['data'];

        return result.map(((e) => LandPropertyBoundaries.fromJson(e))).toList();
      } else {
        throw Exception(response.reasonPhrase);
      }
    } catch (e) {
      throw e.toString();
    }
  }

  Future<http.Response?> update(LandPropertyBoundaries data, id) async {
    String path = Url.instance.getLandPropertyBoundaries();
    Map<String, String> headers = {
      'Content-Type': 'application/json; charset=UTF-8',
      'X-Client-Key': xClientKey,
      'X-Client-Secret': xClientKeySecret
    };
    Map<String, String> params = {
      "landappr_details_id": id.toString(),
    };
    http.Response? response;
    try {
      response = await Request.instance.putRequest(
          path: path, body: data.toJson(), headers: headers, param: params);
    } catch (e) {
      log(e.toString());
    }
    return response;
  }

  Future<http.Response?> updateEdit(LandPropertyBoundaries data, id) async {
    String path = Url.instance.getLandPropertyBoundaries();
    Map<String, String> headers = {
      'Content-Type': 'application/json; charset=UTF-8',
      'X-Client-Key': xClientKey,
      'X-Client-Secret': xClientKeySecret
    };
    Map<String, String> params = {
      "landappr_details_id": id.toString(),
    };
    http.Response? response;
    try {
      response = await Request.instance.putRequest(
          path: path, body: data.toJson(), headers: headers, param: params);
    } catch (e) {
      log(e.toString());
    }
    return response;
  }

  Future<LandPropertyBoundaries> fetchEdit(tempID) async {
    String path = Url.instance.getLandPropertyBoundaries();
    Map<String, String> headers = {
      'Content-Type': 'application/json; charset=UTF-8',
      'X-Client-Key': xClientKey,
      'X-Client-Secret': xClientKeySecret
    };
    Map<String, String> params = {
      "landappr_details_id": tempID.toString(),
    };
    try {
      http.Response response = await Request.instance
          .getRequest(param: params, path: path, headers: headers);

      print('LandBoundEdit');
      print(response.body);

      if (response.statusCode == 200) {
        final jsonData = jsonDecode(response.body);
        final dataList = jsonData['data'] as List<dynamic>;
        final result = LandPropertyBoundaries.fromJson(
            dataList[0] as Map<String, dynamic>);

        return result;
      } else {
        print(response.reasonPhrase);
        throw Exception(response.reasonPhrase);
      }
    } catch (e) {
      throw e.toString();
    }
  }
}
